@@ -7,7 +7,7 @@ warnings.filterwarnings('ignore')
 
 from sklearn.ensemble import RandomForestRegressor,  GradientBoostingRegressor
 from sklearn.ensemble import RandomForestClassifier, GradientBoostingClassifier
-from sklearn.tree import DecisionTreeClassifier, DecisionTreeRegressor
+from sklearn.tree import DecisionTreeClassifier, DecisionTreeRegressor, ExtraTreeRegressor
 from sklearn.linear_model import LogisticRegression, LinearRegression
 from sklearn.kernel_ridge import KernelRidge
 from sklearn.pipeline import make_pipeline
@@ -24,8 +24,11 @@ from sklearn.neighbors import KNeighborsClassifier, KNeighborsRegressor
 from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
 from sklearn.linear_model import Lasso, LassoCV, Ridge, RidgeCV, LassoLarsCV
 from sklearn.ensemble import ExtraTreesClassifier,ExtraTreesRegressor
-from sklearn.ensemble import HistGradientBoostingRegressor, HistGradientBoostingClassifier
+#from sklearn.ensemble import HistGradientBoostingRegressor, HistGradientBoostingClassifier
 from sklearn.multioutput import ClassifierChain, RegressorChain
+from xgboost import XGBClassifier, XGBRegressor
+from lightgbm import LGBMClassifier, LGBMRegressor
+from sklearn.naive_bayes import MultinomialNB, GaussianNB
 
 from sklearn.model_selection import train_test_split
 import pathlib
@@ -54,7 +57,7 @@ class Stacking_Classifier(BaseEstimator, ClassifierMixin, TransformerMixin):
     #################################################################################
     #### Greatly indebted to Gilbert Tanner who explained Stacked Models here
     ####   https://gilberttanner.com/blog/introduction-to-ensemble-learning
-    #### Modifed to create a Stacking Classifier that can handle multi-label targets
+    #### I used the blog to create a Stacking Classifier that can handle multi-label targets
     #################################################################################
     """
     def __init__(self):
@@ -105,10 +108,10 @@ class Stacking_Classifier(BaseEstimator, ClassifierMixin, TransformerMixin):
                 instance.fit(X.iloc[train_index], y.iloc[train_index])
                 y_pred = instance.predict(X.iloc[holdout_index])
                 if y.ndim == 1:
-                    out_of_fold_predictions[holdout_index, i] = y_pred
+                    out_of_fold_predictions[holdout_index, i] = y_pred.ravel()
                 elif y.ndim <= 2:
                     if y.shape[1] == 1:
-                        out_of_fold_predictions[holdout_index, i] = y_pred
+                        out_of_fold_predictions[holdout_index, i] = y_pred.ravel()
                     else:
                         next_i = int(i+self.target_len)
                         out_of_fold_predictions[holdout_index,i:next_i] = y_pred
@@ -159,7 +162,7 @@ class Stacking_Regressor(BaseEstimator, RegressorMixin, TransformerMixin):
     #################################################################################
     #### Greatly indebted to Gilbert Tanner who explained Stacked Models here
     ####   https://gilberttanner.com/blog/introduction-to-ensemble-learning
-    #### Modifed to create a Stacking Regressor that can handle multi-label targets
+    #### I used the blog to create a Stacking Regressor that can handle multi-label targets
     #################################################################################
     """
     def __init__(self, use_features=True):
@@ -172,6 +175,7 @@ class Stacking_Regressor(BaseEstimator, RegressorMixin, TransformerMixin):
         
     def fit(self, X, y):
         """Fit all the models on the given dataset"""
+
         import lightgbm as lgb
         models_dict = stacking_models_list(X_train=X, y_train=y, modeltype='Regression', verbose=1)
         self.base_models = list(models_dict.values())
@@ -199,6 +203,7 @@ class Stacking_Regressor(BaseEstimator, RegressorMixin, TransformerMixin):
         out_of_fold_predictions = np.zeros((X.shape[0], self.target_len*len(self.base_models)))
         for i, model in enumerate(self.base_models):
             print('  %s model training and prediction...' %str(model).split("(")[0])
+            
             start_time = time.time()
             for train_index, holdout_index in kfold.split(X, y):
                 instance = clone(model)
@@ -207,10 +212,10 @@ class Stacking_Regressor(BaseEstimator, RegressorMixin, TransformerMixin):
                 y_pred = instance.predict(X.iloc[holdout_index])
                 
                 if y.ndim == 1:
-                    out_of_fold_predictions[holdout_index, i] = y_pred
+                    out_of_fold_predictions[holdout_index, i] = y_pred.ravel()
                 elif y.ndim <= 2:
                     if y.shape[1] == 1:
-                        out_of_fold_predictions[holdout_index, i] = y_pred
+                        out_of_fold_predictions[holdout_index, i] = y_pred.ravel()
                     else:
                         next_i = int(i+self.target_len)
                         out_of_fold_predictions[holdout_index,i:next_i] = y_pred
@@ -258,7 +263,7 @@ class Blending_Regressor(BaseEstimator, RegressorMixin, TransformerMixin):
     #################################################################################
     #### Greatly indebted to Gilbert Tanner who explained Stacked Models here
     ####   https://gilberttanner.com/blog/introduction-to-ensemble-learning
-    #### Modifed to create a Blending Regressor that can handle multi-label targets
+    #### I used the blog to create a Blending Regressor that can handle multi-label targets
     #################################################################################
     """
     def __init__(self, holdout_pct=0.2, use_features=True):
@@ -297,18 +302,19 @@ class Blending_Regressor(BaseEstimator, RegressorMixin, TransformerMixin):
         else:
             self.target_len = y.shape[1]
 
-        holdout_predictions = np.zeros((X_holdout.shape[0], self.target_len*len(self.base_models)))
+        #holdout_predictions = np.zeros((X_holdout.shape[0], self.target_len*len(self.base_models)))
+        
         for i, model in enumerate(self.base_models_):
             print('  %s model training and prediction...' %str(model).split("(")[0])
             start_time = time.time()
             model.fit(X_train, y_train)
             y_pred = model.predict(X_holdout)
             print('    Time taken = %0.0f seconds' %(time.time()-start_time))
-            
-            if y_train.ndim < 2:
-                holdout_predictions[:, i] = y_pred
+            if i == 0:
+                holdout_predictions = y_pred
             else:
-                holdout_predictions = y_pred[:]
+                holdout_predictions = np.column_stack([holdout_predictions, y_pred])
+        
         if self.use_features:
             if holdout_predictions.ndim < 2:
                 self.meta_model_.fit(np.hstack((X_holdout, holdout_predictions.reshape(-1,1))), y_holdout)
@@ -321,6 +327,7 @@ class Blending_Regressor(BaseEstimator, RegressorMixin, TransformerMixin):
 
     def predict(self, X):
         #### This can handle multi_label predictions now ###
+        
         if self.target_len == 1:
             meta_features = np.column_stack([
                 model.predict(X) for model in self.base_models_])
@@ -395,29 +402,20 @@ def stacking_models_list(X_train, y_train, modeltype='Regression', verbose=0):
                 print('List of models chosen for stacking: %s' %estimators_list)
                 return dict(estimators_list)
         ######    Bagging models if Bagging is chosen ####
-        if y_train.shape[0] <= 10000:
-            model3 = KNeighborsRegressor(n_jobs=-1)
-            estimators.append(('KNN',model3))
-        else:
-            model3 = LinearRegression(n_jobs=-1)
-            estimators.append(('Linear Model',model3))
+        model3 = LinearRegression(n_jobs=-1)
+        estimators.append(('Linear Model',model3))
         ####   Tree models if Linear chosen #####
         model5 = DecisionTreeRegressor(random_state=seed,min_samples_leaf=2)
         estimators.append(('Decision Trees',model5))
         ####   Linear Models if Boosting is chosen #####
-        if y_train.shape[0] <= 10000:
-            model6 = Lasso(alpha=0.01, random_state=seed)
-            estimators.append(('Lasso',model6))
-        else:
-            #model6 = ExtraTreesRegressor(n_estimators=50, random_state=seed)
-            #estimators.append(('Extra Trees Regressor',model6))
-            model6 = HistGradientBoostingRegressor(random_state=seed)
-            estimators.append(('Histogram Gradient Regressor',model6))
+        model6 = ExtraTreeRegressor(random_state=seed)
+        estimators.append(('Extra Tree Regressor',model6))
 
         #model7 = RandomForestRegressor(n_estimators=50,random_state=seed, n_jobs=-1)
         model7 = Ridge(alpha=0.5)
         estimators.append(('Ridge',model7))
     else:
+        ### This is for classification problems ########
         if y_train.ndim >= 2:
             if y_train.shape[1] > 1:
                 stump = lgb.LGBMClassifier(n_estimators=50, random_state=99)
@@ -427,44 +425,26 @@ def stacking_models_list(X_train, y_train, modeltype='Regression', verbose=0):
                 estimator_names = [tuples[0] for tuples in estimators]
                 print('List of models chosen for stacking: %s' %estimators_list)
                 return dict(estimators_list)
+        ### Leave this as it is - don't change it #######
         n_classes = len(Counter(y_train))
         if n_classes > 2:
             model3 = LogisticRegression(max_iter=5000, multi_class='ovr')
-            estimators.append(('Logistic Regression',model3))
         else:
-            if y_train.shape[0] <= 10000:
-                model3 = KNeighborsClassifier(n_jobs=-1)
-                estimators.append(('KNN',model3))
-            else:
-                #model3 = ExtraTreesClassifier(n_estimators=50, random_state=99)
-                #estimators.append(('Extra Trees Classifier',model3))
-                model3 = HistGradientBoostingClassifier(random_state=99)
-                estimators.append(('Histogram Gradient Regressor', model3))
+            model3 = LogisticRegression(max_iter=5000)
+        estimators.append(('Logistic Regression', model3))
         ####   Linear Models if Boosting is chosen #####
-        if n_classes > 2:
-            model4 = RandomForestClassifier(n_estimators=50, random_state=99, n_jobs=-1)
-            estimators.append(('Random Forest Classifier',model4))
-        else:
-            model4 = LinearDiscriminantAnalysis()
-            estimators.append(('Linear Discriminant',model4))
-        
-        ####   Tree models if Linear chosen #####
-        model6 = AdaBoostClassifier(base_estimator=DecisionTreeClassifier(max_depth=3), n_estimators=50)
-        estimators.append(('Adaboost Classifier',model6))
-        
+        model4 = LinearDiscriminantAnalysis()
+        estimators.append(('Linear Discriminant',model4))
+
+        model5 = LGBMClassifier()
+        estimators.append(('LightGBM',model5))
+
         ######    Naive Bayes models if Bagging is chosen ####
-        if n_classes <= 2:
-            try:
-                model7 = GaussianNB()
-            except:
-                model7 = DecisionTreeClassifier(min_samples_leaf=2)
-        else:
-            try:
-                model7 = MultinomialNB()
-            except:
-                model7 = DecisionTreeClassifier(min_samples_leaf=2)
-        estimators.append(('Naive Bayes',model7))
-        
+        model7 = DecisionTreeClassifier(min_samples_leaf=2)
+        estimators.append(('Decision Tree',model7))
+    
+    #### Create a new list here ####################
+
     estimators_list = [(tuples[0],tuples[1]) for tuples in estimators]
     estimator_names = [tuples[0] for tuples in estimators]
     print('List of models chosen for stacking: %s' %estimators_list)
